unit fmHWndTest;

{ Example application for DropMaster.

  Demonstrates accepting drops on a non-Delphi window where we have only a
  window handle.

  Create some instances of Notepad, and use this application to make any one
  of them a text drop target.

  Note - this application is bare-bones;  it doesn't keep track of, e.g., when
  the various instances of Notepad close, etc.

  Last modified:  08/05/2001}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, DMComps;

type

  // Expose the WindowHandle property (see the online help)
  TDummyWinControl = class(TWinControl)
  public
    property WindowHandle;
  end;

  TForm1 = class(TForm)
    Button1: TButton;
    ListBox1: TListBox;
    Button2: TButton;
    DMTextTarget1: TDMTextTarget;
    Label1: TLabel;
    procedure Button1Click(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure DMTextTarget1Drop(Sender: TObject; Acceptor: TWinControl;
      const dropText: String; X, Y: Integer);
    procedure DMTextTarget1DragOver(Sender: TObject; effect: TDropEffect;
      X, Y: Integer);
  private
    { Private declarations }
    FDummy: TDummyWinControl;
    procedure SetAcceptorHWnd(aHWnd: HWnd);
    procedure ClearAcceptorHWnd;
    procedure UpdateButtonAndList;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

uses
  ClipBrd;

{$R *.DFM}


function EnumProc(aHWnd: HWnd; aLParam: integer): BOOL; stdcall;
// Enumeration procedure for EnumWindows below
// Add the title text of all Notepad instances found to the listbox
var
  S: string;
begin
  SetLength(s, 255);
  SetLength(s, GetClassName(aHWnd, PChar(s), 255));
  if (AnsiCompareText(S, 'Notepad') = 0) then
  begin
    SetLength(s, 255);
    SetLength(s, GetWindowText(aHWnd, PChar(s), 255));
    Form1.ListBox1.Items.AddObject(s, TObject(aHWnd));
  end;
  result := true;  // To continue enumeration.
end;

procedure TForm1.Button1Click(Sender: TObject);
// Go find all of the Notepad instances
begin
  ListBox1.Items.Clear;

  EnumWindows(@EnumProc, 0);

  UpdateButtonAndList;
end;

procedure TForm1.UpdateButtonAndList;
begin
  Button2.Enabled := (ListBox1.Items.Count > 0);
  ListBox1.Enabled := Button1.Enabled;
end;

procedure TForm1.SetAcceptorHWnd(aHWnd: HWnd);
// Make aHWnd the drop target
begin
  ClearAcceptorHWnd;

  if (aHWnd <> 0) then
  begin
    FDummy := TDummyWinControl.Create(nil);
    FDummy.WindowHandle := aHWnd;
    DMTextTarget1.AcceptorControl := FDummy;
  end;
end;

procedure TForm1.ClearAcceptorHWnd;
// Clear the drop target.
begin
  DMTextTarget1.AcceptorControl := nil;

  if assigned(FDummy) then
  begin
    FDummy.WindowHandle := 0;
    FDummy.Free;
    FDummy := nil;
  end;
end;

procedure TForm1.FormDestroy(Sender: TObject);
// Cleaning up.  Make sure we kill the drop target.
begin
  ClearAcceptorHWnd;

end;

procedure TForm1.Button2Click(Sender: TObject);
// Make the selected Notepad instance the drop target
var
  aHWnd: HWnd;
begin
  if (ListBox1.ItemIndex >= 0) then
  begin
    aHWnd := THandle(ListBox1.Items.Objects[ListBox1.ItemIndex]);

    // Check whether that Notepad instance might now be gone.
    if IsWindow(aHWnd) then
      aHWnd := FindWindowEx(aHWnd, 0, nil, nil)
    else
    begin
      // Yup, it's gone.  Zero the handle, and delete from the list.
      aHWnd := 0;
      ListBox1.Items.Delete(ListBox1.ItemIndex);
      UpdateButtonAndList;
    end;
      
    SetAcceptorHWnd(aHWnd);

    BringWindowToTop(GetParent(aHWnd));
  end;
end;


procedure TForm1.DMTextTarget1Drop(Sender: TObject; Acceptor: TWinControl;
  const dropText: String; X, Y: Integer);
// We've dropped on an instance of Notepad.  Since the Notepad instance is another
// process, we have no easy way of getting text there (normally, our drop targets are
// in our own process).  Instead, we'll put the dropped text on the clipboard,
// and then paste the clipboard contents into Notepad using the WM_PASTE message.
// We keep a copy of any text on the clipboard and restore it. This preserves any
// text content in the clipboard, but destroys any other content.
var
  S: string;
begin
  Clipboard.Open;
  s := Clipboard.AsText;
  Clipboard.AsText := DMTextTarget1.Text;
  Clipboard.Close;

  // Paste the clipboard text at the current cursor location
  SendMessage(DMTextTarget1.AcceptorControl.Handle, WM_PASTE, 0, 0);

  Clipboard.Open;
  Clipboard.AsText := s;
  Clipboard.Close;
end;

procedure TForm1.DMTextTarget1DragOver(Sender: TObject;
  effect: TDropEffect; X, Y: Integer);
// Make sure the current caret position in the edit control of Notepad is
// kept in sync with the cursor position during the drag.
var
  aCharPos: integer;
begin
  aCharPos := LoWord(SendMessage(DMTextTarget1.AcceptorControl.Handle, EM_CHARFROMPOS, 0, MAKELPARAM(X, Y)));
  SendMessage(DMTextTarget1.AcceptorControl.Handle, EM_SETSEL, aCharPos, aCharPos);
end;

end.
